package com.example.zego_zpns;

import android.content.IntentFilter;

import androidx.annotation.NonNull;

import com.example.zego_zpns.internal.ZPNsPluginEventHandler;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;


import im.zego.zpns.ZPNsManager;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.EventChannel;

/** ZegoZpnsPlugin */
public class ZegoZpnsPlugin implements FlutterPlugin, MethodCallHandler,EventChannel.StreamHandler{
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel methodchannel;

  private final Class<?> manager;

  private ZPNsPluginEventHandler zpnsPluginEventHandler = null;

  private final HashMap<String, Method> methodHashMap = new HashMap<>();

  private FlutterPluginBinding binding = null;

  public ZegoZpnsPlugin() {

    try{
      this.manager = Class.forName("com.example.zego_zpns.internal.ZPNsPluginMethodHandler");
    }catch (ClassNotFoundException e){
      throw new RuntimeException(e);
    }
  }

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    methodchannel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "zego_zpns");
    methodchannel.setMethodCallHandler(this);

    EventChannel eventChannel = new EventChannel(flutterPluginBinding.getBinaryMessenger(),"zpns_event_handler");
    eventChannel.setStreamHandler(this);

    this.binding = flutterPluginBinding;
    this.zpnsPluginEventHandler = new ZPNsPluginEventHandler();
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    if (call.method.equals("getPlatformVersion")) {
      result.success("Android " + android.os.Build.VERSION.RELEASE);
    }
    try{
      Method method = methodHashMap.get(call.method);
      if(method == null){
        if(call.method.equals("registerPush")){
          method = this.manager.getMethod(call.method,MethodCall.class,Result.class,FlutterPluginBinding.class);
        }else {
          method = this.manager.getMethod(call.method, MethodCall.class, Result.class);
        }
        methodHashMap.put(call.method, method);
      }
      if(call.method.equals(("registerPush"))){
        method.invoke(null,call,result,this.binding);
      }
      else {
        method.invoke(null,call, result);
      }
    }
    catch (NoSuchMethodException e){
      result.notImplemented();
    }
    catch (InvocationTargetException e){
      //result.error();
    }
    catch (IllegalAccessException e){
      result.notImplemented();
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    methodchannel.setMethodCallHandler(null);
  }

  @Override
  public void onListen(Object arguments, EventChannel.EventSink events) {
    this.zpnsPluginEventHandler.setSink(events);
  }

  @Override
  public void onCancel(Object arguments) {
    this.zpnsPluginEventHandler.setSink(null);
  }
}
