import 'package:flutter/services.dart';
import 'package:zego_zpns/src/internal/zpns_converter.dart';
import 'package:zego_zpns/src/internal/zpns_event_handler_impl.dart';
import 'package:zego_zpns/zego_zpns.dart';


class ZPNsImpl implements ZPNs {
  static const MethodChannel _channel = MethodChannel('zego_zpns');
  static ZPNsImpl? _zpnsImpl;

  ZPNsImpl() {
    ZPNsEventHandlerImpl.registerEventHandler();
  }

  static ZPNsImpl getInstance() {
    _zpnsImpl ??= ZPNsImpl();

    return _zpnsImpl!;
  }

  @override
  Future<String> getVersion() async {
    final String version = await _channel.invokeMethod('getVersion');
    return version;
  }

  @override
  Future<void> registerPush() async {
    return await _channel.invokeMethod('registerPush');
  }

  @override
  Future<void> enableDebug(bool debug) async {
    return await _channel.invokeMethod('enableDebug', {'debug': debug});
  }

  @override
  Future<ZPNsConfig> getPushConfig() async {
    Map resultMap = await _channel.invokeMethod('getPushConfig');
    return ZPNsConverter.cnvZPNsConfigMapToObject(resultMap['config']);
  }

  @override
  Future<void> setPushConfig(ZPNsConfig config) async {
    return await _channel.invokeMethod(
        'setPushConfig', {'config':ZPNsConverter.cnvZPNsConfigObjectToMap(config)});
  }



  @override
  Future<void> applyNotificationPermission() async {
    return await _channel.invokeMethod('applyNotificationPermission');
  }
}
