import 'package:zego_zpns/src/internal/zpns_impl.dart';
import 'package:zego_zpns/src/zpns_defines.dart';

abstract class ZPNs {
//Common

  /// Available since: 2.1.0 or later.
  ///
  ///Description: Get the ZPNs instance.
  ///
  ///Use cases: This method is used to obtain ZPNs singletons when ZPNs is used.
  static ZPNs getInstance() {
    return ZPNsImpl.getInstance();
  }


  /// Available since: 2.1.0.
  ///
  /// Description: Get the SDK version.
  ///
  /// Use cases:
  /// 1. When the SDK is running, the developer finds that it does not match the expected situation and submits the problem and related logs to the ZEGO technical staff for locating. The ZEGO technical staff may need the information of the engine version to assist in locating the problem.
  /// 2. Developers can also collect this information as the version information of the engine used by the app, so that the SDK corresponding to each version of the app on the line.
  ///
  /// When to call : It can be called at any time.
  Future<String> getVersion();


  ///Available since: 2.1.0 or later.
  ///
  ///Description: This method is used to register vendor offline push.
  ///
  /// Use cases: This method is called when a developer needs to use offline push.
  ///
  /// When to call /Trigger: It can be called when needed. There are no prerequisites.
  Future<void> registerPush();


  ///Available since: 2.1.0 or later.
  ///
  ///Description: This method is used to notify the SDK of the current operating environment, default is false.
  ///
  /// Use cases: This interface is called and set to true when the developer needs to use ZPNs in the debug environment.
  ///
  /// [debug] debug is a bool ,if the current environment is debug,pass true.
  Future<void> enableDebug(bool debug);


/// iOS only
///Available since: 2.1.0 or later.
///
///Description: This method is used to register vendor offline push.
///
/// Use cases: This method is called before needs to use offline push.
///
/// When to call /Trigger: To apply for push permission, note that this method will only pop up one push permission request to the user (if the user does not agree, then the user can only check the corresponding permission in the setting page), and the developer needs to choose an appropriate time to call.
  Future<void> applyNotificationPermission();

  /// Android only
  ///Available since: 2.1.0 or later.
  ///
  ///Description: Set push Settings for each vendor.
  ///
  ///Use cases: Developers need to use this method to set up configurations pushed by vendors.
  ///                                   
  ///Default value: Called when vendor push needs to be set.
  ///[conifg] Add the required configuration items pushed by each vendor and a Boolean value for whether to enable them.
  Future<void> setPushConfig(ZPNsConfig config);

  /// Android only
  ///Available since: 2.1.0 or later.
  ///
  ///Description: Get the PushConfig data class object.
  ///
  ///Use cases: This method is called when the developer needs to get the PushConfig data class object.
  ///
  ///When to call /Trigger: There is no time limit to call, developers can call when they need.
  ///[ZPNsConfig] Gets the latest config.
  Future<ZPNsConfig> getPushConfig();
}
